
/*
 * Primeiro exerccio do Tutorial de J2ME
 *
 * @author Tarcisio Camara
 */
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;

/**
 * Classe abstrata que representa um objeto do jogo.
 * Possui posio x,y onde est na tela; Uma velocidade
 * horizontal e vertical; Um conjunto de imagens que
 * representam graficamente o objetoe mtodos para 
 * deteco de coliso e propriedades do objeto
 */
public abstract class Sprite {

    /** Posio na tela */
    private int           x, y;

    /** Velocidade horizontal e vertical */
    private int           xspeed, yspeed;

    /** Indica se est visvel */
    private boolean       visible;

    /** Representao grfica do objeto */
    private Image[]       image;

    /** Altura do objeto */
    private int height;

    /** Largura do objeto */
    private int width;


    /**
     * Construtor
     *
     * @param x Posio horizontal
     * @param y Posio vertical
     */
    public Sprite(int x, int y) {
        this.x = x;
        this.y = y;
    }

    /**
     * Construtor
     *
     * @param x Posio horizontal
     * @param y Posio vertical
     * @param image Imagem que representa o objeto
     */
    public Sprite(int x, int y, Image[] image) {
        this.image = image;
        this.x = x;
        this.y = y;
        if (this.image[0] == null) {
            this.height = 0;
            this.width = 0;
        } else {
            this.height = this.image[0].getHeight();
            this.width = this.image[0].getWidth();
        }
    }

    /**
     * Verifica interseo entre o objeto e um retngulo
     *
     * @param left Posio esquerda do retngulo
     * @param right Posio direita do retngulo
     * @param top Posio superior do retngulo
     * @param bottom Posio inferior do retngulo
     *
     * @return true Se o retngulo intersecta o objeto
     */
    private boolean rectangleIntersection(int left,
                                          int right,
                                          int top,
                                          int bottom)
    {
        if (getY() + this.getHeight()-1 < top) return(false);
        if (getY() > bottom) return(false);

        if (getX() + this.getWidth()-1 < left) return(false);
        if (getX() > right) return(false);

        return(true);
    }
    
    /**
     * Deve ser implementado pela sub classe
     *
     * @param g The graphics context.
     */
    public abstract  void draw(Graphics g);

    /**
     * Deve ser implementado pela sub classe
     */
    public abstract void update();

    /**
     * Verifca a interseo entre dois Sprites
     *
     * @param GameItem The game item to be tested.
     */
    public boolean intersects(Sprite item) {
        boolean result = false;
        if(this.isVisible()){
            result = rectangleIntersection(item.getX(),
                                            item.getX() +
                                            item.getWidth()-1,
                                            item.getY(),
                                            item.getY() +
                                            item.getHeight()-1
            );
        }
        return result;
    }

    /* ************************************************************/
    /* Mtodos get() e set()
    /* ************************************************************/

    public void setWidth(int width){
        this.width = width;
    }

    public int getWidth(){
        return this.width;
    }

    public void setHeight(int height){
        this.height = height;
    }

    public int getHeight(){
        return this.height;
    }

    public void setImage(Image[] image){
        this.image = image;
    }

    public Image[] getImage(){
        return this.image;
    }

    public int getX() {
        return this.x;
    }

    public int getY() {
        return this.y;
    }

    public void setX(int newX) {
        this.x = newX;
    }

    public void setY(int newY) {
        this.y = newY;
    }

    public int getXspeed() {
        return xspeed;
    }

    public void setXspeed(int newXspeed) {
        this.xspeed = newXspeed;
    }

    public int getYspeed() {
        return yspeed;
    }

    public void setYspeed(int newYspeed) {
        this.yspeed = newYspeed;
    }

    public void setVisible(boolean v){
        this.visible = v;
    }

    public boolean isVisible(){
        return visible;
    }

}
